Attribute VB_Name = "basFileBase"
Option Explicit

Public Declare Function SHFileOperation Lib "shell32.dll" Alias "SHFileOperationA" (lpFileOp As SHFILEOPSTRUCT) As Long

Private Type FILETIME
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type

Public Type SHFILEOPSTRUCT
    hwnd                  As Long
    wFunc                 As Long
    pFrom                 As String
    pTo                   As String
    fFlags                As Integer
    fAnyOperationsAborted As Boolean
    hNameMappings         As Long
    lpszProgressTitle     As String
End Type

Public Type WIN32_FIND_DATA
    dwFileAttributes As Long
    ftCreationTime   As FILETIME
    ftLastAccessTime As FILETIME
    ftLastWriteTime  As FILETIME
    nFileSizeHigh    As Long
    nFileSizeLow     As Long
    dwReserved0      As Long
    dwReserved1      As Long
    cFileName        As String * MAX_PATH
    cAlternate       As String * 14
End Type

Public Const FO_MOVE = &H1
Public Const FO_COPY = &H2
Public Const FO_DELETE = &H3
Public Const FO_RENAME = &H4

Public Const FOF_ALLOWUNDO = &H40
Public Const FOF_MULTIDESTFILES = &H1
Public Const FOF_CONFIRMMOUSE = &H2
Public Const FOF_SILENT = &H4
Public Const FOF_RENAMEONCOLLISION = &H8
Public Const FOF_NOCONFIRMATION = &H10
Public Const FOF_WANTMAPPINGHANDLE = &H20
Public Const FOF_FILESONLY = &H80
Public Const FOF_SIMPLEPROGRESS = &H100
Public Const FOF_NOCONFIRMMKDIR = &H200
Public Const FOF_NOERRORUI As Long = &H400
Public Function SplitPath(ByVal Path As String) As Long
    SplitPath = InStrRev(Path, "\")
End Function
Public Function GetBlankName(ByVal File As String) As String
    Dim x As Integer
        
    x = SplitPath(File)
    File = mID$(File, x + 1)
    
    x = DLL.STRING.InStrRev(File, ".")
    
    If x > 0 Then
        GetBlankName = mID$(File, 1, x - 1)
    Else
        GetBlankName = File
    End If
    
End Function
Public Function GetDir(ByVal Directory As String) As String
    GetDir = Directory
    If Right$(GetDir, 1) <> "\" Then GetDir = GetDir & "\"
End Function
Public Function CutDir(ByVal Directory As String) As String

    Do While Right$(Directory, 1) = "\"
        Directory = mID$(Directory, 1, Len(Directory) - 1)
    Loop

    CutDir = Directory

End Function
Public Function DRVisEqual(ByVal Path1 As String, _
                           ByVal Path2 As String) As Boolean
    
    On Local Error GoTo Quit
    
    Path1 = LCase(GetDir(Path1))
    Path2 = LCase(GetDir(Path2))
    
    DRVisEqual = (mID$(Path1, 1, 3) = mID$(Path2, 1, 3))

Quit:
    Err.Clear
    
End Function
Public Function DeleteTree(ByVal Directory As String) As Long
    
    DeleteTree = DeleteTree_(Directory)
    
End Function
Private Function DeleteTree_(Directory As String) As Long
    Dim sCurrFile As String
    
    On Local Error GoTo Quit
    
    sCurrFile = GetDir(Directory)
    sCurrFile = Dir(sCurrFile & "*.*", vbDirectory)
    
    Do While Len(sCurrFile) > 0
        If sCurrFile <> "." And sCurrFile <> ".." Then
            If (GetAttr(Directory & "\" & sCurrFile) _
            And vbDirectory) = vbDirectory Then
                Call DeleteTree_(Directory & "\" & sCurrFile)
                sCurrFile = Dir(Directory & "\*.*", vbDirectory)
            Else
                Kill Directory & "\" & sCurrFile
                sCurrFile = Dir
            End If
        Else
            sCurrFile = Dir
        End If
    Loop
    
    RmDir Directory
    
Quit:
    DeleteTree_ = Err.Number

    If Err.Number <> 0 Then Call ERROR(basFileBase_, "DeleteTree_", Err.Number, Err.Description)
    
    Err.Clear
    
End Function
Public Function SH_FileOperation(hwnd As Long, _
                                 OPERATION As fOPERATION, _
                                 pFrom As String, _
                                 pTo As String, _
                                 IncludingSubDirectorys As Boolean, _
                                 Confirm As Boolean, _
                                 SHOWDIALOGS As Boolean, _
                                 ShowProgress As Boolean, _
                                 MOVETORECYCLEBIN As Boolean) As Long
    Dim SFO As SHFILEOPSTRUCT
    Dim wFunc As Long, x As Long
    
    On Local Error GoTo Quit
    
    Select Case OPERATION
        Case foCOPY
            wFunc = FO_COPY
        Case foMOVE
            wFunc = FO_MOVE
        Case foRENAME
            wFunc = FO_RENAME
            
            If InStr(pTo, "\") = 0 Then _
                x = SplitPath(pFrom): _
                pTo = GetDir(mID$(pFrom, 1, x - 1)) & pTo
        Case foDELETE
            wFunc = FO_DELETE
            pTo = ""
        Case Else
            Exit Function
    End Select

    With SFO
        .hwnd = hwnd
        .wFunc = wFunc
        .pFrom = pFrom & Chr$(0)
        .pTo = pTo & Chr$(0)
        .fFlags = 0
        
        If Not IncludingSubDirectorys Then
            .fFlags = .fFlags Or FOF_FILESONLY
        Else
            If .wFunc = FO_COPY Then Call DLL.Directory.CreatePath(GetDir(pTo))
        End If
        
        If ShowProgress Then _
            .fFlags = .fFlags Or FOF_SIMPLEPROGRESS
        If Not Confirm Then _
            .fFlags = .fFlags Or FOF_NOCONFIRMATION
        
        If Not SHOWDIALOGS Then _
            .fFlags = .fFlags Or FOF_SILENT: _
            .fFlags = .fFlags Or FOF_NOERRORUI
    
        If OPERATION = foDELETE And MOVETORECYCLEBIN Then _
            .fFlags = .fFlags Or FOF_ALLOWUNDO
    End With

    Call SHFileOperation(SFO)
        
Quit:
    SH_FileOperation = Err.Number
    
    If Err.Number <> 0 Then Call ERROR(basFileBase_, "SH_FileOperation", Err.Number, Err.Description)

    Err.Clear
    
End Function
